JQuest = LibStub("AceAddon-3.0"):NewAddon("JQuest", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("JQuest")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local db
local AddonLibrary = nil

--Frames
local fraJQuestMainAnchor = nil
local dicButtons = {}
local strButtonPrefix = "JQuestButton"
local strHeaderColour = "|cff00aaff"
local strObjectiveColour = "|cffffffff"
local strDoneObjectiveColour = "|cff00ffff"
local strObjectiveDescriptionColour = "|cff888888"
local intNumberOfFrames = 50


local defaults = { --Defaults Table
	profile = {
		Frames = {
			MainWindowX = 0,
			MainWindowY = 0,
			MainWindowPoint = "CENTER",
			MainWindowRelativePoint = "CENTER",
			MainWindowScale = 1,
			MainWindowLocked = false,			
			QuestFontSize = 11,
			HeaderFontSize = 11,
			MinionWidth = 220,
			HeaderColourRed = 0,
			HeaderColourGreen = 0.6,
			HeaderColourBlue = 1,
			HeaderColourAlpha = 1,
			ObjectiveColourRed = 1,
			ObjectiveColourGreen = 1,
			ObjectiveColourBlue = 1,
			ObjectiveColourAlpha = 1,
			ObjectiveDoneColourRed = 0,
			ObjectiveDoneColourGreen = 1,
			ObjectiveDoneColourBlue = 1,
			ObjectiveDoneColourAlpha = 1,		
			ObjectiveDescColourRed = 0.5,
			ObjectiveDescColourGreen = 0.5,
			ObjectiveDescColourBlue = 0.5,
			ObjectiveDescColourAlpha = 1,	
			ShowItemButtons = false,
			HideCompletedObjectives = false,
			ShowNumberOfQuests = false,
			MoveItemsAndTooltipsRight = false,
			ShowDescWhenNoObjectives = false,
		},
	},
}

local function getOptions() --Creates Options table.
	if not JQuest.options then
		JQuest.options = {
			type = "group",
			name = "JQuest",
			handler = JQuest,
			args = {
				General = {
					name = L["General Options"],
					type = "group",
					order = 1,
					args = {
						MainLock = {
							name = L["Lock Frame"],
							desc = L["Unlocks/Locks the Main Frame"],
							type = "toggle",
							get = function() return db.Frames.MainWindowLocked end,
							set = function()
										db.Frames.MainWindowLocked = not db.Frames.MainWindowLocked
										JQuest:doAnchorBackgroundUpdate()
									end,
							order = 1,
						},
						Reset = {
							order = 3,
							type = "execute",
							name = L["Reset Main Frame"],
							desc = L["Resets Main Frame position"],
							func = function()
								db.Frames.MainWindowPoint = "CENTER"
								db.Frames.MainWindowRelativePoint =  "CENTER"
								db.Frames.MainWindowX = 0
								db.Frames.MainWindowY = 0
							
								JQuestMainAnchor:StopMovingOrSizing()		
								JQuestMainAnchor:ClearAllPoints()
								JQuestMainAnchor:SetPoint(db.Frames.MainWindowPoint, UIParent, db.Frames.MainWindowRelativePoint, db.Frames.MainWindowX, db.Frames.MainWindowY)
							end,
						},
						MainWidth = {
							order = 3,
							name = L["Width"],
							desc = L["Controls the width of the main window."],
							type = "range",
							min = 150, max = 300, step = 1,
							isPercent = false,
							get = function() return db.Frames.MinionWidth end,
							set = function(info, value)
								db.Frames.MinionWidth = value
								JQuestMainAnchor:SetWidth(db.Frames.MinionWidth)
								JQuest:UpdateMinion()
							end,
						},	
						ShowNumberQuestsToggle = {
							name = L["Show number of quests"],
							desc = L["Shows/Hides the number of quests"],
							type = "toggle",
							width = "full",
							get = function() return db.Frames.ShowNumberOfQuests end,
							set = function()
								db.Frames.ShowNumberOfQuests = not db.Frames.ShowNumberOfQuests
								JQuest:UpdateMinion()
							end,
							order = 4,
						},
					},
				},
				QuestOptions = {
					name = L["Quest Options"],
					type = "group",
					order = 1,
					args = {
						HeaderFontSize = {
							order = 1,
							name = L["Header Font Size"],
							desc = L["Controls the font size of the header label."],
							type = "range",
							min = 10, max = 20, step = 1,
							isPercent = false,
							get = function() return db.Frames.HeaderFontSize end,
							set = function(info, value)
								db.Frames.HeaderFontSize = value
								JQuest:UpdateMinion()
							end,
						},	
						QuestFontSize = {
							order = 2,
							name = L["Quest Font Size"],
							desc = L["Controls the font size of the quest info."],
							type = "range",
							min = 10, max = 20, step = 1,
							isPercent = false,
							get = function() return db.Frames.QuestFontSize end,
							set = function(info, value)
								db.Frames.QuestFontSize = value
								JQuest:UpdateMinion()
							end,
						},	
						HideCompletedObjectivesToggle = {
							name = L["Hide completed objectives"],
							desc = L["Shows/Hides completed objectives"],
							type = "toggle",
							width = "full",
							get = function() return db.Frames.HideCompletedObjectives end,
							set = function()
								db.Frames.HideCompletedObjectives = not db.Frames.HideCompletedObjectives
								JQuest:UpdateMinion()
							end,
							order = 3,
						},
						ShowItemButtonsToggle = {
							name = L["Show quest item buttons"],
							desc = L["Shows/Hides the quest item buttons"],
							type = "toggle",
							width = "full",
							get = function() return db.Frames.ShowItemButtons end,
							set = function()
								db.Frames.ShowItemButtons = not db.Frames.ShowItemButtons
								JQuest:UpdateMinion()
							end,
							order = 4,
						},
						ItemsAndTooltipsRightToggle = {
							name = L["Display items and tooltips on right"],
							desc = L["Moves items and tooltips to the right"],
							type = "toggle",
							width = "full",
							get = function() return db.Frames.MoveItemsAndTooltipsRight end,
							set = function()
								db.Frames.MoveItemsAndTooltipsRight = not db.Frames.MoveItemsAndTooltipsRight
								JQuest:UpdateMinion()
							end,
							order = 5,
						},
						ShowDescWhenNoObjectivesToggle = {
							name = L["Display quest description if not objectives"],
							desc = L["Displays a quests description if there are no objectives available"],
							type = "toggle",
							width = "full",
							get = function() return db.Frames.ShowDescWhenNoObjectives end,
							set = function()
								db.Frames.ShowDescWhenNoObjectives = not db.Frames.ShowDescWhenNoObjectives
								JQuest:UpdateMinion()
							end,
							order = 6,
						},
					},
				},
				ColourOptions = {
					name = L["Colour Options"],
					type = "group",
					order = 2,
					args = {		
						HeaderColour = {
							name = L["Zone Header Colour"],
							desc = L["Sets the color for the header of each zone"],
							type = "color",
							width = "full",
							hasAlpha = true,
							get = function() return db.Frames.HeaderColourRed, db.Frames.HeaderColourGreen, db.Frames.HeaderColourBlue, db.Frames.HeaderColourAlpha end,
							set = function(_,r,g,b,a)
									db.Frames.HeaderColourRed = r
									db.Frames.HeaderColourGreen = g
									db.Frames.HeaderColourBlue = b
									db.Frames.HeaderColourAlpha = a
									JQuest:doColourChange()
								end,
							order = 1,
						},
						UndoneObjectiveColour = {
							name = L["Incomplete objective colour"],
							desc = L["Sets the color for the incomplete objectives"],
							type = "color",
							width = "full",
							hasAlpha = true,
							get = function() return db.Frames.ObjectiveColourRed, db.Frames.ObjectiveColourGreen, db.Frames.ObjectiveColourBlue, db.Frames.ObjectiveColourAlpha end,
							set = function(_,r,g,b,a)
									db.Frames.ObjectiveColourRed = r
									db.Frames.ObjectiveColourGreen = g
									db.Frames.ObjectiveColourBlue = b
									db.Frames.ObjectiveColourAlpha = a
									JQuest:doColourChange()
								end,
							order = 2,
						},
						DoneObjectiveColour = {
							name = L["Complete objective colour"],
							desc = L["Sets the color for the complete objectives"],
							type = "color",
							width = "full",
							hasAlpha = true,
							get = function() return db.Frames.ObjectiveDoneColourRed, db.Frames.ObjectiveDoneColourGreen, db.Frames.ObjectiveDoneColourBlue, db.Frames.ObjectiveDoneColourAlpha end,
							set = function(_,r,g,b,a)
									db.Frames.ObjectiveDoneColourRed = r
									db.Frames.ObjectiveDoneColourGreen = g
									db.Frames.ObjectiveDoneColourBlue = b
									db.Frames.ObjectiveDoneColourAlpha = a
									JQuest:doColourChange()
								end,
							order = 3,
						},
						NoObjectivesColour = {
							name = L["No objectives description colour"],
							desc = L["Sets the color for the description displayed when there is no quest objectives"],
							type = "color",
							width = "full",
							hasAlpha = true,
							get = function() return db.Frames.ObjectiveDescColourRed, db.Frames.ObjectiveDescColourGreen, db.Frames.ObjectiveDescColourBlue, db.Frames.ObjectiveDescColourAlpha end,
							set = function(_,r,g,b,a)
									db.Frames.ObjectiveDescColourRed = r
									db.Frames.ObjectiveDescColourGreen = g
									db.Frames.ObjectiveDescColourBlue = b
									db.Frames.ObjectiveDescColourAlpha = a
									JQuest:doColourChange()
								end,
							order = 4,
						},
					},
				},
			},
		}
		JQuest.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(JQuest.db)
		JQuest.options.args.profiles.order = 9
	end
	return JQuest.options
	
end

function JQuest:OnInitialize() --Called when the addon is loaded
	self.db = LibStub("AceDB-3.0"):New("JQuestDB", defaults, true)
	db = self.db.profile
	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
	
	self:SetupOptions()
	self:SetupColours()
end

function JQuest:OnEnable() --Called when the addon is enabled
	self:CreateLayout()
	self:RegisterEvent("QUEST_LOG_UPDATE")	
	self:RegisterEvent('PLAYER_LEVEL_UP');
end

function JQuest:RefreshConfig()
	db = self.db.profile
	JQuestMainAnchor:SetWidth(db.Frames.MinionWidth)
	self:doColourChange()
	self:doAnchorBackgroundUpdate()
	self:UpdateMinion()
end

function JQuest:CreateLayout()
	fraJQuestMainAnchor = self:doCreateFrame("FRAME","JQuestMainAnchor",UIParent,db.Frames.MinionWidth,40,1,"BACKGROUND",1,db.Frames.MainWindowPoint, UIParent, db.Frames.MainWindowRelativePoint, db.Frames.MainWindowX, db.Frames.MainWindowY,1)
	fraJQuestMainAnchor:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
	fraJQuestMainAnchor:SetMovable(true)
	fraJQuestMainAnchor:SetClampedToScreen(true)
	fraJQuestMainAnchor:RegisterForDrag("LeftButton")
	fraJQuestMainAnchor:SetScript("OnDragStart", fraJQuestMainAnchor.StartMoving)
	fraJQuestMainAnchor:SetScript("OnDragStop",  function(self)
		fraJQuestMainAnchor:StopMovingOrSizing()
		strPoint, tempB, strRelativePoint, intPosX, intPosY = fraJQuestMainAnchor:GetPoint()
		db.Frames.MainWindowPoint = strPoint
		db.Frames.MainWindowRelativePoint = strRelativePoint
		db.Frames.MainWindowX = intPosX
		db.Frames.MainWindowY = intPosY
	end)
	fraJQuestMainAnchor.objFontString = fraJQuestMainAnchor:CreateFontString(nil, "OVERLAY");
	fraJQuestMainAnchor.objFontString:SetFont("Interface\\AddOns\\JQuest\\Fonts\\framd.ttf", 14)
	fraJQuestMainAnchor.objFontString:SetPoint("TOPLEFT", fraJQuestMainAnchor, "TOPLEFT", 0, 0);
	fraJQuestMainAnchor.objFontString:SetJustifyH("CENTER")
	fraJQuestMainAnchor.objFontString:SetJustifyV("TOP")
	fraJQuestMainAnchor.objFontString:SetText("");
	fraJQuestMainAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
	fraJQuestMainAnchor.objFontString:SetShadowOffset(1, -1)
	
	self:doAnchorBackgroundUpdate()
	self:ShowJQuest()
	
	local strButtonName = ""
	for i=1,intNumberOfFrames,1 do
		strButtonName = strButtonPrefix .. i
		dicButtons[strButtonPrefix .. i] = self:doCreateLooseFrame("BUTTON",strButtonName,JQuestMainAnchor,10,10,1,"LOW",1,1)
		dicButtons[strButtonPrefix .. i].objFontString = dicButtons[strButtonPrefix .. i]:CreateFontString(nil, "OVERLAY");
		dicButtons[strButtonPrefix .. i].objFontString:SetFont("Interface\\AddOns\\JQuest\\Fonts\\framd.ttf", 11)
		dicButtons[strButtonPrefix .. i].objFontString:SetPoint("LEFT", dicButtons[strButtonPrefix .. i], "LEFT", 0, 0);
		dicButtons[strButtonPrefix .. i].objFontString:SetJustifyH("LEFT")
		dicButtons[strButtonPrefix .. i].objFontString:SetJustifyV("TOP")
		dicButtons[strButtonPrefix .. i].objFontString:SetText("");
		dicButtons[strButtonPrefix .. i].objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
		dicButtons[strButtonPrefix .. i].objFontString:SetShadowOffset(1, -1)

		strButtonName = strButtonPrefix .. "item" .. i
		dicButtons[strButtonPrefix .. i].fraItemButton = CreateFrame("BUTTON", strButtonName, dicButtons[strButtonPrefix .. i], "WatchFrameItemButtonTemplate");
		dicButtons[strButtonPrefix .. i].fraItemButton:SetScale(0.75)

		dicButtons[strButtonPrefix .. i].isHeader = nil
		
		dicButtons[strButtonPrefix .. i]:RegisterForClicks("AnyUp")
		dicButtons[strButtonPrefix .. i]:SetScript("OnClick", function(self)
			if (dicButtons[strButtonPrefix .. i].isHeader == 1) then
				if (dicButtons[strButtonPrefix .. i].isCollapsed == 1) then
					ExpandQuestHeader(i)
				else
					CollapseQuestHeader(i)
				end
			else
				if (IsShiftKeyDown()) then
					ChatEdit_InsertLink(GetQuestLink(i))
				else
					ShowUIPanel(QuestLogFrame);
					SelectQuestLogEntry(i);
					QuestLog_SetSelection(i);
					QuestLog_Update();
				end
			end
		end)
		dicButtons[strButtonPrefix .. i]:SetScript("OnEnter", function(self)
			if (not(dicButtons[strButtonPrefix .. i].isHeader)) then
				SelectQuestLogEntry(i);
				local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(i);
				if (db.Frames.MoveItemsAndTooltipsRight == true) then
					GameTooltip:SetOwner(dicButtons[strButtonPrefix .. i], "ANCHOR_RIGHT", 0, -50);
				else 
					GameTooltip:SetOwner(dicButtons[strButtonPrefix .. i], "ANCHOR_LEFT", 0, -50);
				end
				
				local strQuestDescription, strQuestObjectives = GetQuestLogQuestText();
				local objColour = GetQuestDifficultyColor(level);
				GameTooltip:SetText(questTitle, 0, 1, 0, 1);
				GameTooltip:AddLine(strQuestObjectives, 0.5, 0.5, 0.5, 1);
				
				local strQuestTag = ""
				if (isDaily) then
					strQuestTag = strQuestTag .. L["Daily "]
				end				
				if (questTag) then
					strQuestTag = strQuestTag .. questTag
					if (suggestedGroup > 0) then
						strQuestTag = strQuestTag .. " (" .. suggestedGroup .. ")"
					end
				end				
				GameTooltip:AddLine(strQuestTag, 1, 0, 0, 1);
				
				local intPartyMembers = GetNumPartyMembers();
				local blnIsOnQuest = false
				if (intPartyMembers > 0) then
					GameTooltip:AddLine(L["Party members on quest:"], 0, 1, 0, 1);
					for k = 1, intPartyMembers do
						blnIsOnQuest = IsUnitOnQuest(i, "party" .. k);
						if (blnIsOnQuest == 1) then
							GameTooltip:AddLine(UnitName("party" .. k), 1, 1, 1, 1);
						end
					end				
				end
				
				GameTooltip:Show();
			end
		end)
		dicButtons[strButtonPrefix .. i]:SetScript("OnLeave", function(self) 
			GameTooltip:Hide() 
		end)

	end
end

function JQuest:UpdateMinion()
	local fraTempHeaderFrame = fraJQuestMainAnchor
	local intHeaderPosition = 0
	local intQuestOffset = 10
	local intSpacingIncrease = 0

	local numEntries, numQuests = GetNumQuestLogEntries();	
	if (db.Frames.ShowNumberOfQuests == true) then
		fraJQuestMainAnchor.objFontString:SetText(strHeaderColour .. numQuests .. "/25|r");
	else
		fraJQuestMainAnchor.objFontString:SetText("");
	end
	
	for i=1,numEntries,1 do
		local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(i);
		dicButtons[strButtonPrefix .. i]:Show()
		dicButtons[strButtonPrefix .. i]:ClearAllPoints()
		dicButtons[strButtonPrefix .. i].isHeader = isHeader
		dicButtons[strButtonPrefix .. i].isCollapsed = isCollapsed
		dicButtons[strButtonPrefix .. i].fraItemButton:Hide()
		
		if (not isHeader) then
			local strQuestText = self:GetQuestText(level, questTag, suggestedGroup, isDaily, questTitle, isComplete, i)

			dicButtons[strButtonPrefix .. i]:SetWidth(db.Frames.MinionWidth - intQuestOffset*2)
			dicButtons[strButtonPrefix .. i]:SetPoint("TOP",JQuestMainAnchor,"CENTER",intQuestOffset,intHeaderPosition)
			dicButtons[strButtonPrefix .. i].objFontString:SetFont("Interface\\AddOns\\JQuest\\Fonts\\framd.ttf", db.Frames.QuestFontSize)
			dicButtons[strButtonPrefix .. i].objFontString:SetText(strQuestText);
			dicButtons[strButtonPrefix .. i].objFontString:SetWidth(db.Frames.MinionWidth - intQuestOffset*2)
			
			intSpacingIncrease = dicButtons[strButtonPrefix .. i].objFontString:GetHeight()
			dicButtons[strButtonPrefix .. i]:SetHeight(intSpacingIncrease)
			
			if (db.Frames.ShowItemButtons == true) then
				local link, item, charges = GetQuestLogSpecialItemInfo(i);
				if (item) then
					dicButtons[strButtonPrefix .. i].fraItemButton:ClearAllPoints();
					if (db.Frames.MoveItemsAndTooltipsRight == true) then
						dicButtons[strButtonPrefix .. i].fraItemButton:SetPoint("TOPLEFT", dicButtons[strButtonPrefix .. i], "TOPRIGHT", 7, 0)
					else
						dicButtons[strButtonPrefix .. i].fraItemButton:SetPoint("TOPRIGHT", dicButtons[strButtonPrefix .. i], "TOPLEFT", -12, 0)
					end
					dicButtons[strButtonPrefix .. i].fraItemButton:Show()
					dicButtons[strButtonPrefix .. i].fraItemButton:SetID(i)
					SetItemButtonTexture(dicButtons[strButtonPrefix .. i].fraItemButton, item);
					SetItemButtonCount(dicButtons[strButtonPrefix .. i].fraItemButton, charges);
					WatchFrameItem_UpdateCooldown(dicButtons[strButtonPrefix .. i].fraItemButton);
				end
			end
		else
			local strPrefix = strHeaderColour .. "- "
			if (isCollapsed == 1) then
				strPrefix = strHeaderColour .. "+ "
			end	
		
			dicButtons[strButtonPrefix .. i]:SetWidth(db.Frames.MinionWidth)
			dicButtons[strButtonPrefix .. i]:SetPoint("TOP",JQuestMainAnchor,"CENTER",0,intHeaderPosition)
			dicButtons[strButtonPrefix .. i].objFontString:SetFont("Interface\\AddOns\\JQuest\\Fonts\\framd.ttf", db.Frames.HeaderFontSize)
			dicButtons[strButtonPrefix .. i].objFontString:SetText(strPrefix .. questTitle .. "|r");
			dicButtons[strButtonPrefix .. i].objFontString:SetWidth(db.Frames.MinionWidth)
			
			intSpacingIncrease = dicButtons[strButtonPrefix .. i].objFontString:GetHeight()
			dicButtons[strButtonPrefix .. i]:SetHeight(intSpacingIncrease)
		end
		intHeaderPosition = intHeaderPosition - intSpacingIncrease 
    end
	if (numEntries < intNumberOfFrames) then
		for i=numEntries + 1,intNumberOfFrames,1 do
			dicButtons[strButtonPrefix .. i]:Hide()
			dicButtons[strButtonPrefix .. i].fraItemButton:Hide()
		end
	end
	
end

function JQuest:GetQuestText(level, questTag, suggestedGroup, isDaily, questTitle, isComplete, i)
	local objColour = GetQuestDifficultyColor(level);
	local strTitleColor = format("|c%02X%02X%02X%02X", 255, objColour.r * 255, objColour.g * 255, objColour.b * 255);
	local strReturnText = strTitleColor .. "[" .. level
	
	SelectQuestLogEntry(i);
	local numQuestLogLeaderBoards = GetNumQuestLeaderBoards(i)

	if (questTag == LFG_TYPE_DUNGEON) then
		strReturnText = strReturnText .. "d"
	end		
	if (questTag == RAID) then
		strReturnText = strReturnText .. "r"
	end		
	if (questTag == PVP) then
		strReturnText = strReturnText .. "p"
	end		
	if (suggestedGroup > 0) then
		strReturnText = strReturnText .. "g" .. suggestedGroup
	end
	if (isDaily) then
		strReturnText = strReturnText .. "*"
	end
	
	strReturnText = strReturnText .. "] " .. questTitle .. "|r"
	if (isComplete and isComplete < 0) then 
		strReturnText = strReturnText .. L[" (Failed)"]
	elseif (isComplete and isComplete > 0) then
		strReturnText = strReturnText .. L[" (Done)"]
	elseif (numQuestLogLeaderBoards == 0) then
		strReturnText = strReturnText .. L[" (goto)"]
	end
	
	strReturnText = strReturnText .. "\n"

	if (numQuestLogLeaderBoards > 0) then --If has objectives then for each objective print, else print quest 'objective' description
		for k=1,numQuestLogLeaderBoards,1 do
			local leaderboardTxt, itemType, isDone = GetQuestLogLeaderBoard(k,i);
			if (leaderboardTxt ~= nil) then --For some reason some quests have hidden objectives that are blank o_O
				if ((isDone == nil)) then
					strReturnText = strReturnText .. strObjectiveColour .. " - " .. leaderboardTxt .. "|r\n"
				else
					if (db.Frames.HideCompletedObjectives == false) then
						strReturnText = strReturnText .. strDoneObjectiveColour .. " - " .. leaderboardTxt .. "|r\n"
					end
				end	
			end
		end
	else 
		if (db.Frames.ShowDescWhenNoObjectives == true) then
			local strQuestDescription, strQuestObjectives = GetQuestLogQuestText();
			if (not(isComplete and isComplete > 0)) then
				strReturnText = strReturnText .. strObjectiveDescriptionColour .. " - " .. strQuestObjectives .. "|r\n"
			else
				if (db.Frames.HideCompletedObjectives == false) then
					strReturnText = strReturnText .. strObjectiveDescriptionColour .. " - " .. strQuestObjectives .. "|r\n"
				end
			end	
		end
	end
	strReturnText = strtrim(strReturnText)
	return strReturnText
end

function JQuest:QUEST_LOG_UPDATE(...)
	self:UpdateMinion()
end

function JQuest:PLAYER_LEVEL_UP(...)
	self:UpdateMinion()
end

function JQuest:SetupOptions() --Creates options and sets the AddonLibrary variable. Registers chat commands
	AddonLibrary = getOptions()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("JQuest", AddonLibrary)
	
	AceConfigDialog:SetDefaultSize("JQuest", 680,525)
	self:RegisterChatCommand( "jq", "ChatCommand")
	self:RegisterChatCommand( "JQuest", "ChatCommand")
end

function JQuest:ChatCommand(input) --Handles chat commands
	if not input or input:trim() == "" then
		LibStub("AceConfigDialog-3.0"):Open("JQuest")
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(JQuest, "JQuest", "JQuest", input)
	end
end

function JQuest:doColourChange() --Changes the addons colours on the fly as changed
	self:SetupColours()
	self:UpdateMinion()
end

function JQuest:SetupColours()
	strHeaderColour = format("|c%02X%02X%02X%02X", 255, db.Frames.HeaderColourRed * 255, db.Frames.HeaderColourGreen * 255, db.Frames.HeaderColourBlue * 255);
	strObjectiveColour = format("|c%02X%02X%02X%02X", 255, db.Frames.ObjectiveColourRed * 255, db.Frames.ObjectiveColourGreen * 255, db.Frames.ObjectiveColourBlue * 255);
	strDoneObjectiveColour = format("|c%02X%02X%02X%02X", 255, db.Frames.ObjectiveDoneColourRed * 255, db.Frames.ObjectiveDoneColourGreen * 255, db.Frames.ObjectiveDoneColourBlue * 255);
	strObjectiveDescriptionColour = format("|c%02X%02X%02X%02X", 255, db.Frames.ObjectiveDescColourRed * 255, db.Frames.ObjectiveDescColourGreen * 255, db.Frames.ObjectiveDescColourBlue * 255);
end

function JQuest:doAnchorBackgroundUpdate()
	if db.Frames.MainWindowLocked == false then
		JQuestMainAnchor:SetBackdropColor(0, 1, 0, 0.5)
		JQuestMainAnchor:SetBackdropBorderColor(0.5, 0.5, 0, 0)
		JQuestMainAnchor:EnableMouse(true)
	else
		JQuestMainAnchor:SetBackdropColor(0, 1, 0, 0)
		JQuestMainAnchor:SetBackdropBorderColor(0.5, 0.5, 0, 0)
		JQuestMainAnchor:EnableMouse(false)
	end
end

function JQuest:doCreateLooseFrame(strType,strName,fraParent,intWidth,intHeight,intScale,strStrata,intLevel,intAlpha,finherit)
	local f = CreateFrame(strType,strName,fraParent,finherit)
	f:SetWidth(intWidth)
	f:SetHeight(intHeight)
	f:SetFrameStrata(strStrata)
	f:SetFrameLevel(intLevel)	
	f:SetAlpha(intAlpha)
	return f  
end 

function JQuest:doCreateFrame(strType,strName,fraParent,intWidth,intHeight,intScale,strStrata,intLevel,strPoint,fraRelativeFrame,strRelativePoint,intOffsetX,intOffsetY,intAlpha,finherit)
	local f = CreateFrame(strType,strName,fraParent,finherit)
	
	f:SetWidth(intWidth)
	f:SetHeight(intHeight)
	f:SetFrameStrata(strStrata)
	f:SetFrameLevel(intLevel)
	f:SetPoint(strPoint,fraRelativeFrame,strRelativePoint,intOffsetX,intOffsetY)
	f:SetAlpha(intAlpha)
	return f  
end 

function JQuest:ShowJQuest()
	fraJQuestMainAnchor:Show()
end

function JQuest:HideJQuest()
	fraJQuestMainAnchor:Hide()
end